
local LBI = LibStub:GetLibrary("LibBabble-Inventory-3.0"):GetReverseLookupTable()

local TradeSkillLookup = {}

do
  local TradeSkillAch = {
    Cooking = {
      GourmetNorthrend = true,
      GourmetOutland = true
    }
  }
  local lookup, id, name, _, completed
  for tradeName, list in pairs(TradeSkillAch) do
    TradeSkillLookup[tradeName] = {}
    lookup = TradeSkillLookup[tradeName]
    for ach in pairs(list) do
      id = OVERACHIEVER_ACHID[ach]
      for i=1,GetAchievementNumCriteria(id) do
        name, _, completed = GetAchievementCriteriaInfo(id, i)
        if (not completed) then
          lookup[name] = lookup[name] or {}
          lookup[name][id] = i
        end
      end
    end
  end
  TradeSkillAch = nil
end

local list

local function TradeSkillCheck(tradeName, name, getList)
  local lookup = TradeSkillLookup[tradeName][name]
  if (lookup) then
    local anyIncomplete
    if (getList) then  list = list and wipe(list) or {};  end

    for id,i in pairs(lookup) do
      _, _, completed = GetAchievementCriteriaInfo(id, i)
      if (completed) then
        lookup[id] = nil
      else
        if (not getList) then  return id;  end
        anyIncomplete = true
        list[#list+1] = id
      end
    end

    if (anyIncomplete) then  return list;  end
  end
end


local icons, highlights = {}
local currentButton

local ExamineTradeSkillUI

local function skillButtonOnEnter(self, _, calledByExamine)
  currentButton = self
  local icon = icons[self]
  if (icon.name) then
    local achlist = TradeSkillCheck(LBI[GetTradeSkillLine()], icon.name, true)
    if (achlist) then
      GameTooltip:SetOwner(self, "ANCHOR_NONE")
      GameTooltip:SetPoint("TOPLEFT", self, "TOPRIGHT", -45, 0)
      GameTooltip:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
      GameTooltip:AddLine(OVERACHIEVER_STRINGS_REQUIREDFORMETATIP)
      GameTooltip:AddLine(" ")
      Overachiever.AddAchListToTooltip(GameTooltip, achlist)
      GameTooltip:AddLine(" ")
      GameTooltip:Show()
      return true
    elseif (not calledByExamine) then
    -- A criteria must have been earned while the Trade Skills frame was open. Reexamine it:
      ExamineTradeSkillUI()
    end
  end
end

local function skillButtonOnLeave()
  currentButton = nil
  GameTooltip:Hide()
end

local function GetIcon(skillButton)
  local icon = icons[skillButton]
  if (icon) then  return icon;  end
  icon = skillButton:CreateTexture(nil, "OVERLAY")
  icon:SetTexture("Interface\\AddOns\\Overachiever\\AchShield")
  icon:SetWidth(12)
  icon:SetHeight(12)
  icon:SetPoint("LEFT", skillButton, "LEFT", 7, 0)
  icons[skillButton] = icon

  local highlight = skillButton:CreateTexture(nil, "HIGHLIGHT")
  highlight:SetTexture("Interface\\AddOns\\Overachiever_Trade\\AchShieldGlow")
  highlight:SetWidth(12)
  highlight:SetHeight(12)
  highlight:SetPoint("CENTER", icon, "CENTER")
  highlights = highlights or {}
  highlights[icon] = highlight

  local prev = skillButton:GetScript("OnEnter")
  if (prev) then
    skillButton:HookScript("OnEnter", skillButtonOnEnter)
  else
    skillButton:SetScript("OnEnter", skillButtonOnEnter)
  end
  prev = skillButton:GetScript("OnLeave")
  if (prev) then
    skillButton:HookScript("OnLeave", skillButtonOnLeave)
  else
    skillButton:SetScript("OnLeave", skillButtonOnLeave)
  end

  if (MouseIsOver(skillButton)) then
    currentButton = skillButton  -- Causes ExamineTradeSkillUI to trigger skillButtonOnEnter.
  end

  return icon
end

function ExamineTradeSkillUI()
  -- Hide all icons:
  for btn, icon in pairs(icons) do
    icon:Hide()
    highlights[icon]:Hide()
    icon.name = nil
  end

  local tradeName = LBI[GetTradeSkillLine()]
  if (TradeSkillLookup[tradeName]) then
    -- Find icons that should be displayed:
    local skillOffset = FauxScrollFrame_GetOffset(TradeSkillListScrollFrame)
    local skillName, skillType
    for i=1,TRADE_SKILLS_DISPLAYED do
      skillName, skillType = GetTradeSkillInfo(i + skillOffset)
      if (skillName and skillType ~= "header" and TradeSkillCheck(tradeName, skillName)) then
        local icon = GetIcon( _G["TradeSkillSkill"..i] )
        icon:Show()
        highlights[icon]:Show()
        icon.name = skillName
      end
    end
  end

  -- Needed for when the button's contents change while the cursor is over it:
  if (currentButton and not skillButtonOnEnter(currentButton, nil, true)) then
    GameTooltip:Hide()
  end
end

hooksecurefunc("TradeSkillFrame_Update", ExamineTradeSkillUI)


-- Skillet addon support:
if (Skillet) then

  local function sortByAchRequired(tradeName, a, b)
-- This groups recipes that are required by the same number of achievements together. Each group is ordered according
-- to the first achievement (by ID) that requires each recipe, sorting them first by the achievement name, then the
-- achievement ID, then the recipe name. Recipes that are required by the most achievements go first.
    local aName = GetTradeSkillInfo(a)
    local bName = GetTradeSkillInfo(b)
    tradeName = LBI[tradeName]

    if (TradeSkillLookup[tradeName]) then
      -- Get number of achievements that require this recipe:
      a = TradeSkillCheck(tradeName, aName, true)
      local aNum = a and #a or 0
      if (a) then  -- This can't wait since TradeSkillCheck is going to be called again, which wipes the table.
        sort(a)
        a = a[1]
      end
      b = TradeSkillCheck(tradeName, bName, true)
      local bNum = b and #b or 0

      if (aNum ~= bNum) then
        return aNum > bNum  -- Notice that we check greater-than, not less-than here.
      elseif (aNum > 0) then  -- If both are greater than zero:
        sort(b)
        b = b[1]
        local aID, aV = GetAchievementInfo(a)
        local bID, bV = GetAchievementInfo(b)
        if (aID ~= bID) then
          if (aV ~= bV) then
            return aV < bV
          else
            return aID < bID
          end
        end
      end
    end

    -- Fall back to names by alphabetical order:
    return aName < bName
  end

  Skillet:AddRecipeSorter(OVERACHIEVER_STRINGS_TRADE_SKILLET_ACHSORT, sortByAchRequired)

  local orig_get_extra = Skillet.GetExtraItemDetailText
  Skillet.GetExtraItemDetailText = function(obj, tradeskill, skill_index)
    local before = orig_get_extra(obj, tradeskill, skill_index)
    tradeskill = LBI[tradeskill]
    local achs = TradeSkillLookup[tradeskill] and TradeSkillCheck(tradeskill, GetTradeSkillInfo(skill_index), true)
    if (not achs) then  return before;  end

    local myvalue = "|TInterface\\AddOns\\Overachiever_Trade\\AchShieldGlow:0|t |cffffd100" ..
      OVERACHIEVER_STRINGS_REQUIREDFORMETATIP .. "|cffffffff"
    local _, name
    if (#achs > 1) then
      for i,id in ipairs(achs) do
        _, name = GetAchievementInfo(id)
        myvalue = myvalue.."|n  - "..name
      end
    else
      _, name = GetAchievementInfo(achs[1])
      myvalue = myvalue.."  "..name
    end
    myvalue = myvalue.."|r"

    if (before) then
      return before .. "|n" .. myvalue
    else
      return myvalue
    end
  end

  local orig_get_prefix = Skillet.GetRecipeNamePrefix
  Skillet.GetRecipeNamePrefix = function(obj, tradeskill, skill_index)
    local before = orig_get_prefix(tradeskill, skill_index)
    tradeskill = LBI[tradeskill]
    local ach = TradeSkillLookup[tradeskill] and TradeSkillCheck(tradeskill, GetTradeSkillInfo(skill_index))
    if (not ach) then  return before;  end

    local myvalue = "|TInterface\\AddOns\\Overachiever\\AchShield:16:16:-4:-2|t"
    if (before) then
      return myvalue .. before
    else
      return myvalue
    end
  end

end
